# -*- coding: utf-8 -*-
from ckeditor.widgets import CKEditorWidget
from ckeditor_uploader.widgets import CKEditorUploadingWidget
from django import forms
from django.conf import settings
from django.contrib import admin
from django.contrib.admin import TabularInline
from django.core.validators import ValidationError
from django.db.models import TextField
from django.utils.translation import ugettext as _
from image_cropping import ImageCropWidget

from jmb.jadmin import JumboModelAdmin, AjaxInline, register_inline, ConstrainedModelForm
from jmb.showcase import models as showcase_models
from jmb.showcase.admin import ItemAdmin

from isolmant.product import models
from modeltranslation.admin import TranslationAdmin, TabbedTranslationAdmin

from image_cropping import ImageCroppingMixin

log_fieldset = (
    (_('logs admin'), {
        'fields': (
            'creator', 'date_create',
            'last_modifier', 'date_last_modify',
        ),
        'classes': ('collapse',)
    })
)
ordering_fieldsets = (
    (_('ordering admin'), {
        'fields': ('ordering',),
        'classes': ('collapse',)
    })
)
seo_fieldsets = (
    (_('SEO admin'), {
        'fields': (
            'meta_description', 'meta_keywords',
            'page_title',
        ),
        'classes': ('collapse',)
    })
)
status_fieldsets = (
    (_('status'), {
        'fields': (
            'status',
        ),
        'classes': ('collapse',)
    })
)

class ItemMixinForm(ConstrainedModelForm):

    def clean(self):
        form_data = self.cleaned_data
        if form_data['slug']:
            pk = self.instance.pk if self.instance.pk else None
            if self.Meta.model.objects.filter(slug=form_data['slug']).exclude(pk=pk):
                self._errors['slug'] = self.error_class([_("Slug isn't unique")])
                del form_data['slug']
        return form_data


class CategoryMediaInline(TabularInline):
    model = models.Media.categories.through


class CategoryAdmin(JumboModelAdmin, TranslationAdmin):
    list_display = (
        'get_edit_icon', 'ordering', 'name', 'slug', 'color'
    )
    inlines = [CategoryMediaInline]
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'name', 'slug',
                'description',
                'color', 'logo',
            )
        })
    )
    tabs = (
        (_('Main'), {
            'items': [_('main')]
        }),
        (_('Medias'), {
            'items': [CategoryMediaInline]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    prepopulated_fields = {"slug": ("name",)}


class ApplicationMediaInline(TabularInline):
    model = models.Media.applications.through


class ApplicationAdminForm(forms.ModelForm):
    class Meta:
        model = models.Application
        fields = '__all__'

    def clean_slug(self):
        slug = self.cleaned_data['slug']
        category = self.cleaned_data['category']
        # adds category slug as prefix to application slug
        slug = "{}-{}".format(category.slug, slug)
        return slug


class ApplicationAdmin(JumboModelAdmin, TranslationAdmin):
    inlines = [ApplicationMediaInline]
    form = ApplicationAdminForm
    list_display = (
        'get_edit_icon', 'ordering', 'category', 'name', 'slug',
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'category',
                'name', 'slug',
                'description',
            )
        })
    )
    tabs = (
        (_('Main'), {
            'items': [_('main')]
        }),
        (_('Media'), {
            'items': [ApplicationMediaInline]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    prepopulated_fields = {"slug": ("name",)}


class SubCategoryMediaInline(TabularInline):
    model = models.Media.subcategories.through


class SubCategoryAdmin(JumboModelAdmin, TranslationAdmin):
    inlines = [SubCategoryMediaInline]
    list_display = (
        'get_edit_icon', 'ordering', 'category', 'application', 'name', 'slug'
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'application',
                ('name_it', 'slug'),
                'description_it',
                'name_en',
                'description_en',
            )
        })
    )
    tabs = (
        (_('Main'), {
            'items': [_('main')]
        }),
        (_('Media'), {
            'items': [SubCategoryMediaInline]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    prepopulated_fields = {"slug": ("name",)}


class ProductVersionInline(AjaxInline):
    model = models.ProductVersion
    fk_name = "product"
    list_display = (
        'name', 'thickness',
        'slug', 'get_edit_icon', 'get_delete_icon',
    )


class ProductVersionForm(ConstrainedModelForm):
    class Meta:
        model = models.ProductVersion
        exclude = []
    hidden_fields = ('product', 'slug',)


class ProductVersionAdmin(JumboModelAdmin, TranslationAdmin):
    form = ProductVersionForm
    tabs = (
        (_('Main*'), {
            'items': [_('format')]
        }),
        (_('Acustic properties'), {
            'items': [_('acustic_properties')]
        }),
        (_('Mechanical properties'), {
            'items': [_('mechanical_properties')]
        }),
        (_('Thermal properties'), {
            'items': [_('thermal_properties')]
        }),
        (_('Packaging*'), {
            'items': [_('packaging')]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    advanced_search_fields = (
        ('product__subcategory__application__category:category',),
        ('product__subcategory__application:application',),
        ('product__subcategory:subcategory',),
        ('product__name__icontains:name', 'thickness'),
    )
    list_display = (
        'get_edit_icon', 'name', 'thickness',
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('format'), {
            'fields': settings.PRODUCT_PRODUCTVERSION_FORMAT_FIELDS
        }),
        (_('acustic_properties'), {
            'fields': settings.PRODUCT_PRODUCTVERSION_ACUSTIC_FIELDS
        }),
        (_('mechanical_properties'), {
            'fields': settings.PRODUCT_PRODUCTVERSION_MECHANICAL_FIELDS
        }),
        (_('thermal_properties'), {
            'fields': settings.PRODUCT_PRODUCTVERSION_THERMAL_FIELDS
        }),
        (_('packaging'), {
            'fields': settings.PRODUCT_PRODUCTVERSION_PACKAGING_FIELDS
        }),
    )


class ProductMediaInline(TabularInline):
    model = models.Media.products.through


class ProductAdmin(ImageCroppingMixin, JumboModelAdmin, TranslationAdmin):
    inlines = [ProductMediaInline]
    tabs = (
        (_('Product'), {
            'items': [_('main'), _('description')]
        }),
        (_('Italian'), {
            'items': [_('description it')]
        }),
        (_('English'), {
            'items': [_('description en')]
        }),
        (_('Versions'), {
            'items': [ProductVersionInline]
        }),
        (_('Accessories'), {
            'items': [_('accessories')]
        }),
        (_('Media'), {
            'items': [ProductMediaInline]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    advanced_search_fields = (
        ('subcategory__application:application',),
        ('subcategory__application__category:category', 'subcategory',),
        ('name__icontains',),
    )
    list_display = (
        'get_edit_icon', 'ordering', 'name',
        'category', 'subcategory', 'application'
    )
    list_display_csv = (
        'ordering', 'name', 'category', 'subcategory', 'application', 'description'
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                ('name', 'slug',),
                ('subcategory',),
                'image', 'cropping',
            )
        }),
        (_('description it'), {
            'fields': (
                'italian_visible',
                'description_it', 'abstract_it',
                'areas_of_use_it', 'instruction_it'
            )
        }),
        (_('description en'), {
            'fields': (
                'english_visible',
                'description_en', 'abstract_en',
                'areas_of_use_en', 'instruction_en'
            )
        }),
        (_('accessories'), {
            'fields': ('accessories',)
        }),
    )
    filter_horizontal = ('accessories',)
    prepopulated_fields = {"slug": ("name",)}


class ProductLabelAdmin(JumboModelAdmin, TranslationAdmin):
    list_display = (
        'get_edit_icon', 'field_name', 'get_label'
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'field_name',
                'label',
            )
        })
    )
    formfield_overrides = {
        TextField: {'widget': CKEditorWidget},
    }

    def get_label(self, obj):
        return u'{}'.format(obj.label)
    get_label.allow_tags = True


class MediaAdmin(JumboModelAdmin, TranslationAdmin):
    advanced_search_fields = (
        ('typology',),
        ('categories', 'subcategories', 'products'),
    )
    tabs = (
        (_('media'), {
            'items': [_('main')]
        }),
        (_('italian'), {
            'items': [_('italian')]
        }),
        (_('english'), {
            'items': [_('english')]
        }),
        (_('Others'), {
            'items': [_('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, status_fieldsets,
        (_('italian'), {
            'fields': (
                'title_it',
                'description_it',
                ('media_it', 'url_it',),
            )
        }),
        (_('english'), {
            'fields': (
                'title_en',
                'description_en',
                ('media_en', 'url_en',),
            )
        }),
        (_('main'), {
            'fields': (
                'typology', 'categories', 'subcategories', 'products',
                'references', 'applications', 'solutions',
            )
        }),
    )
    filter_horizontal = (
        'categories', 'subcategories', 'products',
        'references', 'applications', 'solutions',
    )
    list_display = (
        'get_edit_icon',
        'get_title', 'typology', 'str_category_names', 'str_subcategory_names',
        'str_reference_names', 'str_application_names',
        'str_solution_names',
        'get_media_url',
    )

    def get_title(self, obj):
        if obj.title:
            return obj.title
        elif obj.title_it:
            return obj.title_it
        elif obj.title_en:
            return obj.title_en
        else:
            return 'media'
    get_title.short_description = _('Title')


class SolutionMediaInline(TabularInline):
    model = models.Media.solutions.through


class SolutionAdmin(JumboModelAdmin, TranslationAdmin):
    inlines = [SolutionMediaInline]
    list_display = (
        'get_edit_icon', 'ordering', 'get_category_name', 'application',
        'name_it', 'slug_it', 'name_en', 'slug_en',
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('italian'), {
            'fields': (
                'italian_visible',
                ('name_it', 'slug_it',),
                'description_it',
            )
        }),
        (_('english'), {
            'fields': (
                'english_visible',
                ('name_en', 'slug_en',),
                'description_en',
            )
        }),
        (_('main'), {
            'fields': (
                'application',
                'products'
            )
        })
    )
    tabs = (
        (_('Main'), {
            'items': [_('main')]
        }),
        (_('italian'), {
            'items': [_('italian')]
        }),
        (_('english'), {
            'items': [_('english')]
        }),
        (_('Media'), {
            'items': [SolutionMediaInline]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    prepopulated_fields = {"slug_it": ("name_it",),  "slug_en": ("name_en",)}
    filter_horizontal = ('products',)


class AdvancedFilterForm(ConstrainedModelForm):
    class Meta:
        model = models.AdvancedFilter
        exclude = []

    def clean(self):
        cleaned_data = super(AdvancedFilterForm, self).clean()
        if cleaned_data['filter_type'] == settings.PRODUCT_ADVANCEDFILTER_FILTERTYPE_DICT['range']:
            if not cleaned_data['lower']:
                self.add_error(
                    'lower',
                    _('If filter type is range, lower field is required')
                )
            if not cleaned_data['higher']:
                self.add_error(
                    'higher',
                    _('If filter type is range, higher field is required')
                )


class AdvancedFilterAdmin(JumboModelAdmin):
    form = AdvancedFilterForm
    list_display = (
        'get_edit_icon', 'category', 'label', 'attr', 'get_filter_type_display',
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'category',
                'filter_type',
                'attr',
                'label',
                ('lower', 'higher'),
            )
        })
    )


class ReferenceMediaInline(TabularInline):
    model = models.Media.references.through


class ReferenceAdmin(ImageCroppingMixin, JumboModelAdmin):
    list_display = (
        'get_edit_icon', 'name', 'slug', 'get_products_list'
    )
    inlines = [ReferenceMediaInline]
    tabs = (
        (_('reference'), {
            'items': [_('main')]
        }),
        (_('italian'), {
            'items': [_('italian')]
        }),
        (_('english'), {
            'items': [_('english')]
        }),
        (_('medias'), {
            'items': [ReferenceMediaInline]
        }),
        (_('Others'), {
            'items': [_('SEO admin'), _('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, seo_fieldsets, status_fieldsets,
        (_('italian'), {
            'fields': (
                ('name_it', 'slug',),
                'description_it',
            )
        }),
        (_('english'), {
            'fields': (
                'name_en',
                'description_en',
            )
        }),
        (_('main'), {
            'fields': (
                'image', 'cropping',
                'products', 'reference_category'
            )
        })
    )
    prepopulated_fields = {"slug": ("name_it",)}
    filter_horizontal = ('products',)


class ReferenceCategoryAdmin(JumboModelAdmin):
    list_display = (
        'get_edit_icon', 'name', 'color', 'logo'
    )
    tabs = (
        (_('main'), {
            'items': [_('main')]
        }),
        (_('Others'), {
            'items': [_('ordering admin'), _('logs admin'), _('status')]
        }),
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'name_it', 'name_en',
                'color', 'logo'
            )
        }),
    )


class PressAdminForm(ItemMixinForm):
    class Meta:
        model = models.Press
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class PressAdmin(ImageCroppingMixin, JumboModelAdmin):
    search_fields = ('title',)
    form = PressAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                ('author', 'source'), 'language', 'content', 'image', 'small_image', 'file', 'link', 'publication_date'
            )
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class NewsAdminForm(ItemMixinForm):
    class Meta:
        model = models.News
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class NewsAdmin(ImageCroppingMixin, JumboModelAdmin):

    search_fields = ('title',)
    form = NewsAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                'language', 'content', 'image', 'small_image', 'large_image', 'link', 'publication_date'
            )
        }),
        (_('seo admin'), {
            'fields': (
                'meta_description', 'meta_keywords',
                'page_title',),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class BlogAdminForm(ItemMixinForm):
    class Meta:
        model = models.Blog
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class BlogAdmin(ImageCroppingMixin, JumboModelAdmin):
    search_fields = ('title',)
    form = BlogAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                'language', 'content', 'image', 'small_image', 'large_image', 'link', 'publication_date'
            )
        }),
        (_('seo admin'), {
            'fields': (
                'meta_description', 'meta_keywords',
                'page_title',),
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class HighlightAdminForm(ItemMixinForm):
    class Meta:
        model = models.Highlight
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class HighlightAdmin(ImageCroppingMixin, JumboModelAdmin):
    search_fields = ('title',)
    form = HighlightAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                'language', 'content', 'image', 'small_image', 'link', 'publication_date'
            )
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class HomePresentationAdminForm(ItemMixinForm):
    class Meta:
        model = models.HomePresentation
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class HomePresentationAdmin(ImageCroppingMixin, JumboModelAdmin):
    search_fields = ('title',)
    form = PressAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                'language', 'content', 'image', 'link', 'publication_date'
            )
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class ConventionAdminForm(ItemMixinForm):
    class Meta:
        model = models.Convention
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class ConventionAdmin(ImageCroppingMixin, JumboModelAdmin):
    search_fields = ('title',)
    form = ConventionAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                ('author', 'source'), 'language', 'content', 'image', 'small_image', 'file', 'link', 'publication_date'
            )
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class CourseAdminForm(ItemMixinForm):
    class Meta:
        model = models.Course
        exclude = []
        widgets = {
            'content': CKEditorUploadingWidget
        }


class CourseAdmin(ImageCroppingMixin, JumboModelAdmin):
    search_fields = ('title',)
    form = CourseAdminForm
    advanced_search_fields = (
        ('title__icontains', 'content__icontains'),
        ('tale__icontains',),
        ('publication_date__lte', 'publication_date__gte'),
        ('link_type',),
        ('categories',),
    )
    fieldsets = (
        (None, {
            'fields': (
                'status', ('title', 'slug'), 'categories',
            )
        }),
        (_('Content'), {
            'fields': (
                ('author', 'source'), 'language', 'content', 'image', 'small_image', 'file', 'link', 'publication_date'
            )
        }),
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_("Ordering"), {
            'fields': ('ordering',),
            'classes': ('collapse',)
        }),
    )
    list_display = (
        'get_status_icon', 'ordering', 'title', 'slug',
        'publication_date', 'get_categories', 'language',
        'get_edit_icon', 'get_delete_icon',
    )
    save_on_top = True
    prepopulated_fields = {"slug": ("title",)}
    filter_horizontal = ('categories',)


class GalleryAdmin(JumboModelAdmin):
    list_display = (
        'get_edit_icon', 'name', 'slug',
    )
    tabs = (
        (_('main'), {
            'items': [_('main')]
        }),
        (_('Others'), {
            'items': [_('ordering admin'), _('logs admin'), _('status'), _('SEO admin')]
        }),
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                'name', 'slug', 'description'
            )
        }),
    )


class GalleryImageAdmin(ImageCroppingMixin, JumboModelAdmin):
    list_display = (
        'get_edit_icon', 'name', 'gallery'
    )
    tabs = (
        (_('main'), {
            'items': [_('main')]
        }),
        (_('Others'), {
            'items': [_('ordering admin'), _('logs admin'), _('status'), _('SEO admin')]
        }),
    )
    fieldsets = (
        log_fieldset, ordering_fieldsets, status_fieldsets,
        (_('main'), {
            'fields': (
                ('name', 'slug'),
                'gallery',
                'image',
                'cropping',
                'thumbnail'
            )
        }),
    )


class ItemAdminForm(forms.ModelForm):
    class Meta:
        widgets = {
            'small_image': ImageCropWidget,
            'rollover_small_image': ImageCropWidget,
            'content': CKEditorUploadingWidget
        }


class ItemCkeditorAdmin(ItemAdmin):
    form = ItemAdminForm


register_inline(ProductVersionInline)
admin.site.register(models.Product, ProductAdmin)
admin.site.register(models.ProductLabel, ProductLabelAdmin)
admin.site.register(models.ProductVersion, ProductVersionAdmin)
admin.site.register(models.Category, CategoryAdmin)
admin.site.register(models.Application, ApplicationAdmin)
admin.site.register(models.SubCategory, SubCategoryAdmin)
admin.site.register(models.Media, MediaAdmin)
admin.site.register(models.Solution, SolutionAdmin)
admin.site.register(models.AdvancedFilter, AdvancedFilterAdmin)
admin.site.register(models.Reference, ReferenceAdmin)
admin.site.register(models.Press, PressAdmin)
admin.site.register(models.News, NewsAdmin)
admin.site.register(models.Blog, BlogAdmin)
admin.site.register(models.Highlight, HighlightAdmin)
admin.site.register(models.HomePresentation, HomePresentationAdmin)
admin.site.register(models.Convention, ConventionAdmin)
admin.site.register(models.Course, CourseAdmin)
admin.site.register(models.ReferenceCategory, ReferenceCategoryAdmin)
admin.site.register(models.Gallery, GalleryAdmin)
admin.site.register(models.GalleryImage, GalleryImageAdmin)
admin.site.unregister(showcase_models.Item)
admin.site.register(showcase_models.Item, ItemCkeditorAdmin)
