# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from easy_thumbnails.files import get_thumbnailer
from image_cropping import ImageRatioField

from cms.models import CMSPlugin
from cms.utils.compat.dj import python_2_unicode_compatible

from jmb.core.db.cmsmodels import StatusModel

from isolmant.product.models import Gallery


@python_2_unicode_compatible
class FilterSettings(CMSPlugin, StatusModel):
    template = models.CharField(
        _("template"),
        max_length=255,
        choices=settings.PRODUCT_CMSPLUGIN_FILTER_TEMPLATES,
        default="product/cms_plugins/category_filter.html"
    )
    model_class = models.CharField(
        _("model class"),
        max_length=255,
        null=True, blank=True,
        choices=settings.PRODUCT_CMSPLUGIN_FILTER_MODEL_CLASSES,
        default="isolmant.product.models.Category",
    )

    def __str__(self):
        return "{} - {}".format(
            ('On' if self.status else 'Off'),
            self.get_model_class_display(),
        )

    class Meta:
        app_label = "product"


@python_2_unicode_compatible
class ListSettings(CMSPlugin, StatusModel):
    template = models.CharField(
        _("template"),
        max_length=255,
        choices=settings.PRODUCT_CMSPLUGIN_LIST_TEMPLATES,
        default="product/cms_plugins/product_list.html"
    )
    model_class = models.CharField(
        _("model class"),
        max_length=255,
        choices=settings.PRODUCT_CMSPLUGIN_LIST_MODEL_CLASSES,
        default="isolmant.product.models.Product",
    )

    def __str__(self):
        return "{} - {}".format(
            ('On' if self.status else 'Off'),
            self.get_model_class_display(),
        )

    class Meta:
        app_label = "product"


@python_2_unicode_compatible
class MenuButtonSettings(CMSPlugin, StatusModel):
    button_text = models.CharField(
        max_length=255,
        verbose_name=_('text'),
    )
    button_url = models.CharField(
        max_length=255,
        verbose_name=_('link'),
        help_text=_("For internal links, use relative paths. For anchors, use the anchor name, preceded #")
    )
    color = models.CharField(
        max_length=6,
        verbose_name=_('color'),
        help_text=_('Insert the desired color in hex (ex. 00ff00). Some templates may not support color.')
    )
    template = models.CharField(
        verbose_name=_("template"),
        max_length=255,
        choices=settings.PRODUCT_CMSPLUGIN_MENU_BUTTON_TEMPLATES,
        default="product/cms_plugins/squared_menu_button.html"
    )

    class Meta:
        app_label = "product"
        verbose_name = _("Menu Button Setting")

    def __str__(self):
        return u'{} {}'.format(('On - ' if self.status else 'off - '), self.button_text)


@python_2_unicode_compatible
class GallerySettings(CMSPlugin, StatusModel):
    gallery = models.ForeignKey(
        Gallery,
        verbose_name=_(u'Gallery'),
        related_name='gallery_plugin'
    )
    allow_fullscreen = models.CharField(
        max_length=30,
        verbose_name=_(u'allow fullscreen'),
        help_text=_(u'check to allow fullscreen'),
        choices=settings.PRODUCT_CMSPLUGIN_GALLERY_ALLOWFULLSCREEN,
    )
    hash = models.BooleanField(
        verbose_name=_(u'hash'),
    )
    loop = models.BooleanField(
        verbose_name=_(u'loop'),
    )
    thumbnails = models.BooleanField(
        verbose_name=_(u'thumbnails'),
    )
    caption = models.BooleanField(
        verbose_name=_(u'Caption'),
        default=False
    )
    autoplay = models.IntegerField(
        verbose_name=_(u'autoplay'),
        help_text=_(u'Second for slide autoplay. If 0 disables autoplay.'),
        default=0,
    )
    fit = models.CharField(
        max_length=30,
        verbose_name=_(u'fit'),
        help_text=_(u'how to fit image'),
        choices=settings.PRODUCT_CMSPLUGIN_GALLERY_FIT,
    )

    class Meta:
        app_label = "product"
        verbose_name = _("Gallery Setting")

    def __str__(self):
        return u'{0} {1} {2}'.format(_(u'Gallery'), ('On - ' if self.status else 'Off - '), self.gallery.name)

    def get_autoplay(self):
        return self.autoplay * 1000


@python_2_unicode_compatible
class SliderSettings(CMSPlugin, StatusModel):
    autoplay = models.IntegerField(
        verbose_name=_(u'autoplay'),
        help_text=_(u'Second for slide autoplay. If 0 disables autoplay.'),
        default=0,
    )
    fit = models.CharField(
        max_length=30,
        verbose_name=_(u'fit'),
        help_text=_(u'how to fit image'),
        choices=settings.PRODUCT_CMSPLUGIN_GALLERY_FIT,
    )

    class Meta:
        app_label = "product"

    def __str__(self):
        return u'{0} {1}'.format(_(u'Slider'), ('On' if self.status else 'Off'))

    def get_autoplay(self):
        return self.autoplay * 1000

    def copy_relations(self, oldinstance):
        # Before copying related objects from the old instance, the ones
        # on the current one need to be deleted. Otherwise, duplicates may
        # appear on the public version of the page
        self.slider_items.all().delete()

        for slider_item in oldinstance.slider_items.all():
            # instance.pk = None; instance.pk.save() is the slightly odd but
            # standard Django way of copying a saved model instance
            links = slider_item.links.all()
            slider_item.pk = None
            slider_item.slug += 'prod'
            slider_item.plugin = self
            slider_item.save()
            for link in links:
                slider_item.links.add(link)


@python_2_unicode_compatible
class LinkItem(models.Model):
    title = models.CharField(
        max_length=255,
        verbose_name=_("title")
    )
    link = models.URLField(
        blank=True,
        verbose_name=_("link")
    )
    ordering = models.IntegerField(
        default=0,
        verbose_name=_('ordering'),
    )

    def __str__(self):
        return u'{0} {1}'.format(_(u'Link'), self.title)

    class Meta:
        app_label = "product"
        ordering = ("ordering", "title")


@python_2_unicode_compatible
class SliderItem(models.Model,):
    plugin = models.ForeignKey(
        SliderSettings,
        related_name="slider_items",
        verbose_name=_("plugin")
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
    )
    title = models.CharField(
        verbose_name=_('title'),
        blank=True,
        max_length=255,
    )
    subtitle = models.CharField(
        verbose_name=_('subtitle'),
        blank=True,
        max_length=255,
    )
    image = models.ImageField(
        upload_to='slides',
        verbose_name=_('image'),
    )
    cropping = ImageRatioField(
        'image',
        settings.PRODUCT_GALLERYIMAGE_CROPPING_RATIO_FULL,
        free_crop=True
    )
    links = models.ManyToManyField(
        LinkItem,
        blank=True,
        related_name="slides",
        verbose_name=_('links'),
    )
    ordering = models.IntegerField(
        default=0,
        verbose_name=_('ordering'),
    )

    def get_cropped_full_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': settings.PRODUCT_GALLERYIMAGE_CROPPING_RATIO_FULL.split('x'),
                'box': self.cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False

    def __str__(self):
        return u'{0} {1}'.format(_(u'Slide'), self.title)

    class Meta:
        app_label = "product"
        ordering = ("ordering", "title")


@python_2_unicode_compatible
class MetaTagSettings(CMSPlugin, StatusModel):
    page_title = models.CharField(
        max_length=255,
        blank=True, null=True,
        help_text=_("overwrite the title (html title tag)"),
        verbose_name=_("title"),
    )
    meta_description = models.TextField(
        max_length=255,
        blank=True, null=True,
        help_text=_("The text displayed in search engines."),
        verbose_name=_("description"),
    )
    meta_keywords = models.TextField(
        max_length=255,
        blank=True, null=True,
        help_text=_("The keywords displayed in search engines."),
        verbose_name=_("keywords"),
    )

    def __str__(self):
        return u'{0} {1}'.format(_(u'Meta '), self.page_title)

    class Meta:
        app_label = "product"
