# coding: utf-8
from collections import OrderedDict

from easy_thumbnails.files import get_thumbnailer

import os

from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models import Q
from django.utils.text import slugify
from django.utils.translation import ugettext_lazy as _

import bleach
from ckeditor.fields import RichTextField
from image_cropping import ImageRatioField
from imagekit.models import ImageSpecField, ProcessedImageField
from pilkit.processors import ResizeToFill, ResizeToFit

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.showcase.models import Item


def get_upload_path(instance, filename):
    # get name w/o extension
    file_name = os.path.splitext(filename)[0]
    media_dir = slugify(file_name)[0:3]
    return os.path.join(u"product_files", u"{0}".format(instance.typology), u"{0}".format(media_dir), filename)


class Category(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        unique=True,
        max_length=255,
    )
    # Descrizione
    description = RichTextField(
        verbose_name=_('description'),
    )
    color = models.CharField(
        verbose_name=_('color RGB'),
        max_length=6,
    )
    logo = models.ImageField(
        upload_to='categories',
        verbose_name=_('logo'),
        blank=True,
    )

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')
        ordering = ('ordering', 'name',)

    def __unicode__(self):
        return self.name


class Application(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True,
    )
    # Descrizione
    description = RichTextField(
        verbose_name=_('description'),
    )
    category = models.ForeignKey(
        Category,
        related_name='applications'
    )

    class Meta:
        verbose_name = _('application')
        verbose_name_plural = _('applications')
        ordering = (
            'category__ordering', 'category__name',
            'ordering', 'name',
        )
        unique_together = (('category', 'name',),)

    def __unicode__(self):
        return self.name


class SubCategory(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255
    )
    # Descrizione
    description = RichTextField(
        verbose_name=_('description'),
    )
    application = models.ForeignKey(
        Application,
        related_name='subcategories'
    )

    class Meta:
        verbose_name = _('subcategory')
        verbose_name_plural = _('subcategories')
        ordering = (
            'application__category__ordering', 'application__category__name',
            'application__ordering', 'application__name',
            'ordering', 'name',
        )

    def __unicode__(self):
        return u"{} ({})".format(self.name, self.application.name)

    @property
    def category(self):
        return self.application.category


class Product(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True
    )
    # Descrizione
    description = RichTextField(
        verbose_name=_('description'),
    )
    # Voce di capitolato
    abstract = RichTextField(
        verbose_name=_('abstract'),
    )
    # Settore d'impiego
    areas_of_use = RichTextField(
        verbose_name=_('areas of use'),
    )
    # modalità di applicazione
    instruction = RichTextField(
        verbose_name=_('instruction'),
        blank=True,
    )
    is_accessory = models.BooleanField(
        default=False,
        verbose_name=_('is accessory')
    )

    # RELATIONSHIP FIELDS
    subcategory = models.ForeignKey(
        SubCategory,
        related_name='products',
        verbose_name=_('subcategory'),
    )
    accessories = models.ManyToManyField(
        "self",
        symmetrical=False,
        related_name='products',
        limit_choices_to={'is_accessory': True},
        blank=True,
        verbose_name=_('accessories'),
    )
    image = ProcessedImageField(
        verbose_name="Square thumbnail",
        upload_to='products/images',
        processors=[ResizeToFit(width=800)],
        format='JPEG',
        options={'quality': 60},
        null=True,
        blank=True
    )
    cropping = ImageRatioField('image', '400x250', size_warning=True)
    thumbnail_admin = ImageSpecField(
        source='image',
        processors=[ResizeToFill(width=320, height=200)],
        format='JPEG',
        options={'quality': 60}
    )
    italian_visible = models.BooleanField(
        default=True,
        verbose_name=_('italian visible')
    )
    english_visible = models.BooleanField(
        default=True,
        verbose_name=_('english visible')
    )
    css_class = models.TextField(
        blank=True, null=True,
        max_length=255,
        verbose_name=_('css class')
    )

    @property
    def category(self):
        return self.subcategory.application.category

    @property
    def application(self):
        return self.subcategory.application

    class Meta:
        verbose_name = _('product')
        verbose_name_plural = _('products')
        ordering = (
            "subcategory__application__category__ordering",
            "subcategory__application__category__name",
            "subcategory__application__ordering",
            "subcategory__application__name",
            "subcategory__ordering",
            "subcategory__name",
            'ordering',
            "name",
        )

    def __unicode__(self):
        return self.name

    def save(self, *args, **kwargs):
        self.css_class = self.get_product_css_class()
        super(Product, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('product_detail', args=[self.slug])

    def get_product_css_class(self):
        css_class = '{0} {1} {2}'.format(self.category.slug, self.application.slug, self.subcategory.slug)
        for adv_filter in self.category.advanced_filters.all():
            if adv_filter.filter_type == settings.PRODUCT_ADVANCEDFILTER_FILTERTYPE_DICT['range']:
                css_class += self.get_range_filter_css_class(
                    attr=adv_filter.attr,
                    lower=adv_filter.lower,
                    higher=adv_filter.higher,
                )
            else:
                css_class += self.get_boolean_filter_css_class(attr=adv_filter.attr)
        return css_class

    def get_range_filter_css_class(self, attr='thickness', lower=5, higher=10):
        css_class = ''
        filter_low = {'{0}__{1}'.format(attr, 'lt'): lower}
        filter_mid = {'{0}__{1}'.format(attr, 'gte'): lower, '{0}__{1}'.format(attr, 'lte'): higher}
        filter_high = {'{0}__{1}'.format(attr, 'gt'): higher}
        if self.versions.filter(**filter_low):
            css_class += ' {0}-low'.format(attr)
        if self.versions.filter(**filter_mid):
            css_class += ' {0}-mid'.format(attr)
        if self.versions.filter(**filter_high):
            css_class += ' {0}-high'.format(attr)
        return css_class

    def get_boolean_filter_css_class(self, attr):
        css_class = ''
        if attr:
            filter_dict = {'{}'.format(attr): ''}
            if self.versions.exclude(**filter_dict):
                css_class = ' {0}-true'.format(attr)
            else:
                css_class = ' {0}-false'.format(attr)
        return css_class

    def get_videos(self):
        return Media.objects.filter(
            products=self,
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['video']
        )

    @staticmethod
    def get_cms_qs():
        return Product.objects.filter(status=1).prefetch_related(
            "subcategory__application__category",
            "subcategory__application",
            "subcategory",
        )[:9]

    @staticmethod
    def get_cms_qs_it():
        return Product.objects.filter(status=1, italian_visible=True).prefetch_related(
            "subcategory__application__category",
            "subcategory__application",
            "subcategory",
        )

    @staticmethod
    def get_cms_qs_en():
        return Product.objects.filter(status=1, english_visible=True).prefetch_related(
            "subcategory__application__category",
            "subcategory__application",
            "subcategory",
        )


class ProductVersion(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    product = models.ForeignKey(
        Product,
        related_name="versions",
        verbose_name=_("product"),
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True,
        blank=True, null=True,
    )
    # spessore in mm
    thickness = models.DecimalField(
        verbose_name=_('thickness in mm (approx.)'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # larghezza formato in metri
    format_width = models.DecimalField(
        verbose_name=_('width in m'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # spessore formato in metri
    format_length = models.DecimalField(
        verbose_name=_('length in m'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    format_height = models.DecimalField(
        verbose_name=_('height in m'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # diametro formato
    format_diameter = models.DecimalField(
        verbose_name=_('diameter in cm'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    format_type = models.CharField(
        verbose_name=_('format type'),
        max_length=1,
        choices=settings.PRODUCT_PRODUCT_FORMAT_TYPE_CHOICES,
        blank=True,
    )
    format_weight = models.DecimalField(
        verbose_name=_('weight in kg/m^2'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    packaging_type = models.CharField(
        verbose_name=_('packaging type'),
        max_length=2,
        choices=settings.PRODUCT_PRODUCT_PACKAGING_TYPE_CHOICES,
        blank=True,
    )
    packaging_height = models.DecimalField(
        verbose_name=_('packaging height in cm'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    packaging_width = models.DecimalField(
        verbose_name=_('packaging width in cm'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    packaging_length = models.DecimalField(
        verbose_name=_('packaging length in cm'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    packaging_quantity = models.DecimalField(
        verbose_name=_('packaging quantity'),
        decimal_places=2,
        max_digits=10,
        default=1,
        blank=True, null=True,
    )
    packaging_unity = models.CharField(
        verbose_name=_('unity of measurement for packaging quantity'),
        max_length=2,
        choices=settings.PRODUCT_PRODUCT_PACKAGING_UNITY_CHOICES,
        blank=True,
    )

    # CARATTERISTICHE MECCANICHE
    # Deformazione viscosa (creep)
    creep = models.CharField(
        verbose_name=_('creep'),
        max_length=255,
        blank=True,
    )
    # rigidità dinamica
    # es. s' = x MN/m^3
    dynamic_rigidity = models.DecimalField(
        verbose_name=_('dynamic rigidity in MN/m^3'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # deformazione sotto carico
    deformation = models.CharField(
        verbose_name=_('deformation under load'),
        max_length=255,
        blank=True
    )
    # classe di comprimibilità
    compressibility = models.CharField(
        verbose_name=_('compressibility'),
        max_length=255,
        blank=True
    )
    # Resistenza / Cedimento a compressione
    compression_strain = models.CharField(
        verbose_name=_('compression strain'),
        max_length=255,
        blank=True
    )
    # Densità
    # es. 77kg/m^3
    density = models.DecimalField(
        verbose_name=_('density in kg/m^3'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # Comportamento a lungo termine
    long_term_behaviour = models.CharField(
        verbose_name=_('long term behaviour'),
        max_length=255,
        blank=True
    )
    # Resistenza all'impatto
    # es. 900 mm
    impact_resistance = models.DecimalField(
        verbose_name=_('impact resistance in mm'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # Curva di decompressione
    stress_curve = models.CharField(
        verbose_name=_('Stress-Strain curve'),
        max_length=255,
        blank=True
    )
    # carico di rottura
    breaking_strength = models.CharField(
        verbose_name=_('Breaking strength'),
        max_length=255,
        blank=True, null=True
    )
    # peso in kg per m2
    weight = models.DecimalField(
        verbose_name=_('weight (kg*m2)'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # Conformabilità
    punctual_conformability = models.DecimalField(
        verbose_name=_('punctual conformability'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )

    # CARATTERISTICHE TERMICHE
    # es. R_t = 0,0142 m^2K/W
    thermal_resistance = models.DecimalField(
        verbose_name=_('thermal resistance in m^2K/W'),
        decimal_places=4,
        max_digits=10,
        blank=True, null=True,
    )
    # Conducibilità termica
    # es. lambda = 0,035 W/mK
    thermal_conductivity = models.DecimalField(
        verbose_name=_('thermal conductivity in W/mK'),
        decimal_places=4,
        max_digits=10,
        blank=True, null=True,
    )
    # Calore specifico
    # es. c = 1200 j/kgK
    specific_heat = models.DecimalField(
        verbose_name=_('specific heat in j/kgK'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    reaction_to_fire = models.CharField(
        verbose_name=_('reaction to fire'),
        max_length=255,
        blank=True
    )
    # Spessore equivalente d'aria
    # es. Sd = 0,6m
    equivalent_air_thickness = models.DecimalField(
        verbose_name=_('equivalent air thickness in m'),
        decimal_places=4,
        max_digits=10,
        blank=True, null=True,
    )
    # Fattore di resistenza al vapore
    # mi = 7.000
    vapour_permeability = models.DecimalField(
        verbose_name=_('vapour permeability in mi'),
        decimal_places=2,
        max_digits=10,
        blank=True, null=True,
    )
    # trasmittanza termica
    # es. U_t = 0,461 W^2K
    heat_transfer_coefficient = models.DecimalField(
        verbose_name=_('heat transfer coefficient in W/m^2K'),
        decimal_places=4,
        max_digits=10,
        blank=True, null=True,
    )

    # CARATTERISTICHE ACUSTICHE
    # Impact noise reduction
    # es. DeltaL_w = 25,5 dB
    noise_reduction = models.DecimalField(
        verbose_name=_('impact noise reduction in dB'),
        decimal_places=2,
        max_digits=5,
        blank=True, null=True,
    )
    # Drum sound reduction
    # es. >30 sone
    drum_sound_reduction = models.DecimalField(
        verbose_name=_('drum sound reduction in sone'),
        decimal_places=2,
        max_digits=5,
        blank=True, null=True,
    )
    # potere fonoisolante
    soundproofing_capacity = models.DecimalField(
        verbose_name=_('soundproofing capacity in dB'),
        decimal_places=2,
        max_digits=5,
        blank=True, null=True,
    )
    #
    acoustic_performance_of_small_elements = models.CharField(
        verbose_name=_('Acoustic performance of small elements'),
        max_length=255,
        blank=True
    )
    # Assorbimento acustico
    sound_absorption = models.CharField(
        verbose_name=_('sound absorption'),
        max_length=255,
        blank=True
    )

    class Meta:
        verbose_name = _('product version')
        verbose_name_plural = _('product versions')
        ordering = (
            'ordering',
            "product__subcategory__application__category__name",
            "product__subcategory__name",
            "product__name",
            "thickness",
        )

    def __unicode__(self):
        if self.thickness:
            thick = '%.6g' % self.thickness
            return u"{0} - {1} mm".format(self.name, thick)
        return u"{}".format(self.name)

    @property
    def category(self):
        return self.product.category

    @property
    def subcategory(self):
        return self.product.subcategory

    @property
    def application(self):
        return self.product.application

    @property
    def name(self):
        return self.product.name

    def get_values_dict(self, fields):
        """
        Returns dictionary of fields (key: value)
        :param fields: iterable of fields names
        """
        pv_dict = OrderedDict()
        for field in fields:
            value = getattr(self, field, '')
            if value:
                if hasattr(self, 'get_{}_display'.format(field)):
                    value = getattr(self, 'get_{}_display'.format(field))()
                if hasattr(value, 'is_zero'):
                    value = '%.6g' % value
                try:
                    label = ProductLabel.objects.get(field_name=field).label
                except ProductLabel.DoesNotExist:
                    label = self._meta.get_field(field).verbose_name
                pv_dict[label] = value
        return pv_dict

    def get_format_values_dict(self):
        """
        Returns dictionary of format fields (key: value)
        """
        return self.get_values_dict(settings.PRODUCT_PRODUCTVERSION_FORMAT_FIELDS)

    def get_acustic_values_dict(self):
        """
        Returns dictionary of acustic fields (key: value)
        """
        return self.get_values_dict(settings.PRODUCT_PRODUCTVERSION_ACUSTIC_FIELDS)

    def get_mechanical_values_dict(self):
        """
        Returns dictionary of mechanical fields (key: value)
        """
        return self.get_values_dict(settings.PRODUCT_PRODUCTVERSION_MECHANICAL_FIELDS)

    def get_thermal_values_dict(self):
        """
        Returns dictionary of thermal fields (key: value)
        """
        return self.get_values_dict(settings.PRODUCT_PRODUCTVERSION_THERMAL_FIELDS)

    def get_packaging_values_dict(self):
        """
        Returns dictionary of packaging fields (key: value)
        """
        return self.get_values_dict(settings.PRODUCT_PRODUCTVERSION_PACKAGING_FIELDS)


class ProductLabel(DateModel, UserModel, OrderedModel, StatusModel):
    field_name = models.CharField(
        verbose_name=_('field name'),
        max_length=255,
        db_index=True
    )
    label = models.TextField(
        verbose_name=_('label'),
        blank=True
    )

    class Meta:
        verbose_name = _('Product label')
        verbose_name_plural = _('Product labels')
        ordering = ('field_name',)


class Solution(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True,
    )
    description = RichTextField(
        verbose_name=_('description'),
    )
    application = models.ForeignKey(
        Application,
        related_name='solutions'
    )
    products = models.ManyToManyField(
        Product,
        related_name='solution',
        verbose_name=_('product')
    )
    italian_visible = models.BooleanField(
        default=True,
        verbose_name=_('italian visible')
    )
    english_visible = models.BooleanField(
        default=True,
        verbose_name=_('english visible')
    )

    class Meta:
        verbose_name = _('solution')
        verbose_name_plural = _('solutions')
        ordering = (
            'application__category__ordering', 'application__category__name',
            'application__ordering', 'application__name',
            'ordering', 'name',
        )

    def __unicode__(self):
        return self.name

    def get_category_name(self):
        return self.application.category.name
    get_category_name.short_description = _('category')

    @staticmethod
    def get_cms_qs():
        return Solution.objects.filter(status=1)

    @staticmethod
    def get_cms_qs_it():
        return Solution.objects.filter(status=1, italian_visible=True)

    @staticmethod
    def get_cms_qs_en():
        return Solution.objects.filter(status=1, english_visible=True)


class AdvancedFilter(DateModel, UserModel, OrderedModel, StatusModel):
    category = models.ForeignKey(
        Category,
        related_name='advanced_filters',
        verbose_name=_('category')
    )
    filter_type = models.CharField(
        max_length=3,
        verbose_name=_('filter type'),
        choices=settings.PRODUCT_ADVANCEDFILTER_FILTERTYPE_CHOICES
    )
    label = models.CharField(
        max_length=255,
        verbose_name=_('label'),
    )
    attr = models.CharField(
        max_length=255,
        verbose_name=_('filtering attribute')
    )
    lower = models.DecimalField(
        verbose_name=_('lower'),
        decimal_places=6,
        max_digits=16,
        blank=True, null=True
    )
    higher = models.DecimalField(
        verbose_name=_('higher'),
        decimal_places=6,
        max_digits=16,
        blank=True, null=True
    )

    class Meta:
        verbose_name = _('advanced filter')
        verbose_name_plural = _('advanced filters')
        ordering = (
            'category__ordering', 'category__name',
            'ordering', 'label',
        )

    def __unicode__(self):
        return _("{0} - {1} filter").format(self.category, self.attr)

    @property
    def is_boolean(self):
        if self.filter_type == settings.PRODUCT_ADVANCEDFILTER_FILTERTYPE_DICT['boolean']:
            return True
        else:
            return False

    @property
    def is_range(self):
        if self.filter_type == settings.PRODUCT_ADVANCEDFILTER_FILTERTYPE_DICT['range']:
            return True
        else:
            return False


class ReferenceCategory(DateModel, UserModel, OrderedModel, StatusModel):
    name = models.CharField(
        verbose_name=_(u'name'),
        max_length=255,
    )
    color = models.CharField(
        verbose_name=_(u'color RGB'),
        max_length=6,
        default='CD2C21'
    )
    logo = models.CharField(
        verbose_name=_(u'logo'),
        max_length=64,
        blank=True,
        default='book',
        help_text=_(u"""
            Used to reference one of bootstrap glyphicon. Insert only the specific name
            (ie. for glyphicon-asterisk just write asterisk)
        """)
    )

    class Meta:
        verbose_name = _('reference category')
        verbose_name_plural = _('reference categories')
        ordering = (
            'ordering', 'name',
        )

    def __unicode__(self):
        return self.name


class Reference(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True,
    )
    description = RichTextField(
        verbose_name=_('description'),
    )
    products = models.ManyToManyField(
        Product,
        related_name='references',
        verbose_name=_('product'),
        blank=True
    )
    image = ProcessedImageField(
        verbose_name="Square thumbnail",
        upload_to='references/images',
        processors=[ResizeToFit(width=800)],
        format='JPEG',
        options={'quality': 60},
        null=True,
        blank=True
    )
    cropping = ImageRatioField('image', '400x250', size_warning=True)
    thumbnail_admin = ImageSpecField(
        source='image',
        processors=[ResizeToFill(width=100, height=100)],
        format='JPEG',
        options={'quality': 60}
    )
    reference_category = models.ForeignKey(
        ReferenceCategory,
        related_name='references',
        verbose_name=_('reference category'),
        blank=True, null=True

    )

    class Meta:
        verbose_name = _('reference')
        verbose_name_plural = _('references')
        ordering = ('ordering', 'name',)

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        """
        TODO: da implemetare dopo aver sviluppato Url e View
        """
        return ""

    def get_products_list(self):
        return u', '.join([u'{}'.format(product) for product in self.products.filter(status=1)])
    get_products_list.short_description = 'Products list'

    def get_cleaned_description(self):
        return bleach.clean(
            self.description, tags=[], attributes=[],
            styles=[], strip=True
        )

    def get_attachments(self):
        return self.medias.exclude(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['photo'])

    def get_photos(self):
        return self.medias.filter(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['photo'])


class Media(DateModel, UserModel, OrderedModel, StatusModel):
    title = models.CharField(
        blank=True,
        max_length=255,
        verbose_name=_('title')
    )
    description = RichTextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )
    typology = models.CharField(
        verbose_name=_('typology'),
        max_length=3,
        choices=settings.PRODUCT_MEDIA_TYPOLOGY_CHOICES
    )
    categories = models.ManyToManyField(
        Category,
        related_name='medias',
        blank=True,
        verbose_name=_('categories'),
    )
    subcategories = models.ManyToManyField(
        SubCategory,
        related_name='medias',
        blank=True,
        verbose_name=_('subcategories'),
    )
    products = models.ManyToManyField(
        Product,
        related_name='medias',
        blank=True,
        verbose_name=_('products'),
    )
    references = models.ManyToManyField(
        Reference,
        related_name='medias',
        blank=True,
        verbose_name=_('references')
    )
    applications = models.ManyToManyField(
        Application,
        related_name='medias',
        blank=True,
        verbose_name=_('applications')
    )
    solutions = models.ManyToManyField(
        Solution,
        related_name='medias',
        blank=True,
        verbose_name=_('solutions')
    )
    media = models.FileField(
        upload_to=get_upload_path,
        blank=True,
        verbose_name=_('media'),
    )
    url = models.URLField(
        blank=True,
        verbose_name=_('url'),
    )

    class Meta:
        verbose_name = _('product media')
        verbose_name_plural = _('product media')
        ordering = ('ordering', 'typology', 'title')

    def __unicode__(self):
        title = self.title_it if self.title_it else self.title_en
        return u"{0}: {1}".format(
             self.get_typology_display(),
             title
        )

    def list_category_names(self):
        """
        Returns list of category names that use this media
        """
        name_list = []
        for obj in self.categories.all():
            name_list.append(obj.name)
        return name_list

    def list_subcategory_names(self):
        """
        Returns list of subcategory names that use this media
        """
        name_list = []
        for obj in self.subcategories.all():
            name_list.append(obj.name)
        return name_list

    def list_product_names(self):
        """
        Returns list of product names that use this media
        """
        name_list = []
        for obj in self.products.all():
            name_list.append(obj.name)
        return name_list

    def list_reference_names(self):
        """
        Returns list of reference names that use this media
        """
        name_list = []
        for obj in self.references.all():
            name_list.append(obj.name)
        return name_list

    def list_application_names(self):
        """
        Returns list of application names that use this media
        """
        name_list = []
        for obj in self.applications.all():
            name_list.append(obj.name)
        return name_list

    def list_solution_names(self):
        """
        Returns list of solution names that use this media
        """
        name_list = []
        for obj in self.solutions.all():
            name_list.append(obj.name)
        return name_list

    def str_category_names(self):
        """
        Returns list of category names separated by comma
        """
        if self.list_category_names():
            return ", ".join(self.list_category_names())
        return ''

    def str_subcategory_names(self):
        """
        Returns list of subcategory names separated comma
        """
        if self.list_subcategory_names():
            return ", ".join(self.list_subcategory_names())
        return ''

    def str_product_names(self):
        """
        Returns list of product names separated comma
        """
        if self.list_product_names():
            return ", ".join(self.list_product_names())
        return ''

    def str_reference_names(self):
        """
        Returns list of reference names separated comma
        """
        if self.list_reference_names():
            return ", ".join(self.list_reference_names())
        return ''

    def str_application_names(self):
        """
        Returns list of application names separated comma
        """
        if self.list_application_names():
            return ", ".join(self.list_application_names())
        return ''

    def str_solution_names(self):
        """
        Returns list of solution names separated comma
        """
        if self.list_solution_names():
            return ", ".join(self.list_solution_names())
        return ''

    def get_media_url(self):
        """
        Returns Media url
        """
        if self.url:
            return u"{}".format(self.url)
        elif self.media:
            return u"{}".format(self.media.url)
        return ''

    def media_extension(self):
        """
        Returns file extension uploaded
        """
        name, extension = os.path.splitext(self.media.name)
        return extension

    @staticmethod
    def get_cms_qs():
        return Media.objects.filter(status=1)

    @staticmethod
    def get_cms_qs_it():
        return Media.objects.filter(status=1).exclude(title_it__isnull=True).exclude(title_it='')

    @staticmethod
    def get_cms_qs_en():
        return Media.objects.filter(status=1).exclude(title_en__isnull=True).exclude(title_en='')


class DownloadMedia(Media):
    class Meta:
        proxy = True

    @staticmethod
    def get_cms_qs():
        return Media.objects.filter(status=1).filter(
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['download']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['catalogue']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['brochure']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['price_list'])
        )

    @staticmethod
    def get_cms_qs_it():
        return Media.objects.filter(status=1).exclude(title_it__isnull=True).exclude(title_it='').filter(
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['download']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['catalogue']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['brochure']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['price_list'])
        )

    @staticmethod
    def get_cms_qs_en():
        return Media.objects.filter(status=1).exclude(title_en__isnull=True).exclude(title_en='').filter(
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['download']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['catalogue']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['brochure']) |
            Q(typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['price_list'])
        )


class TechnicalBrochureMedia(Media):
    class Meta:
        proxy = True

    @staticmethod
    def get_cms_qs():
        return Media.objects.filter(status=1).filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['technical_paper']
        )

    @staticmethod
    def get_cms_qs_it():
        return Media.objects.filter(status=1).exclude(title_it__isnull=True).exclude(title_it='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['technical_paper']
        )

    @staticmethod
    def get_cms_qs_en():
        return Media.objects.filter(status=1).exclude(title_en__isnull=True).exclude(title_en='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['technical_paper']
        )


class VideoMedia(Media):
    class Meta:
        proxy = True

    @staticmethod
    def get_cms_qs():
        return Media.objects.filter(status=1).filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['video']
        )

    @staticmethod
    def get_cms_qs_it():
        return Media.objects.filter(status=1).exclude(title_it__isnull=True).exclude(title_it='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['video']
        )

    @staticmethod
    def get_cms_qs_en():
        return Media.objects.filter(status=1).exclude(title_en__isnull=True).exclude(title_en='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['video']
        )


class StandardMedia(Media):
    class Meta:
        proxy = True


class TutorialMedia(Media):
    class Meta:
        proxy = True

    @staticmethod
    def get_cms_qs():
        return Media.objects.filter(status=1).filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['tutorial']
        )

    @staticmethod
    def get_cms_qs_it():
        return Media.objects.filter(status=1).exclude(title_it__isnull=True).exclude(title_it='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['tutorial']
        )

    @staticmethod
    def get_cms_qs_en():
        return Media.objects.filter(status=1).exclude(title_en__isnull=True).exclude(title_en='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['tutorial']
        )


class LegislationMedia(Media):
    class Meta:
        proxy = True

    @staticmethod
    def get_cms_qs():
        return Media.objects.filter(status=1).filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['legislation']
        )

    @staticmethod
    def get_cms_qs_it():
        return Media.objects.filter(status=1).exclude(title_it__isnull=True).exclude(title_it='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['legislation']
        )

    @staticmethod
    def get_cms_qs_en():
        return Media.objects.filter(status=1).exclude(title_en__isnull=True).exclude(title_en='').filter(
            typology=settings.PRODUCT_MEDIA_TYPOLOGY_DICT['legislation']
        )


class PressManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(categories__slug__icontains='press')


class Press(Item):
    class Meta:
        proxy = True
        verbose_name = _('Press release')
        verbose_name_plural = _('Press releases')
        ordering = ('-ordering', '-publication_date', 'title')

    def get_absolute_url(self):
        return "xxx/"

    @staticmethod
    def get_cms_qs():
        return Press.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = PressManager()


class NewsManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(
            categories__slug__icontains='news'
        ).order_by(
            '-publication_date'
        )


class News(Item):
    class Meta:
        proxy = True
        verbose_name = _('News')
        verbose_name_plural = _('News')
        ordering = ('-ordering', '-publication_date', 'title')

    def get_absolute_url(self):
        return "xxx/"

    @staticmethod
    def get_cms_qs():
        return News.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = NewsManager()


class BlogManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(
            categories__slug__icontains='blog'
        ).order_by(
            '-publication_date'
        )


class Blog(Item):
    class Meta:
        proxy = True
        verbose_name = _('Blog')
        verbose_name_plural = _('blog')
        ordering = ('-ordering', '-publication_date', 'title')

    def get_absolute_url(self):
        return "xxx/"

    @staticmethod
    def get_cms_qs():
        return Blog.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = BlogManager()


class HighlightManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(
            Q(categories__slug__icontains='highlight') |
            Q(categories__slug__icontains='highlight-designer') |
            Q(categories__slug__icontains='highlight-dealer') |
            Q(categories__slug__icontains='highlight-builder') |
            Q(categories__slug__icontains='highlight-diy')
        )


class Highlight(Item):
    class Meta:
        proxy = True
        verbose_name = _('Highlight')
        verbose_name_plural = _('Highlights')
        ordering = ('-ordering', '-publication_date', 'title')

    def get_absolute_url(self):
        return "xxx/"

    @staticmethod
    def get_cms_qs():
        return Highlight.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = HighlightManager()


class HomePresentationManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(
            Q(categories__slug__icontains='home-presentation') |
            Q(categories__slug__icontains='home-presentation-designer') |
            Q(categories__slug__icontains='home-presentation-dealer') |
            Q(categories__slug__icontains='home-presentation-builder') |
            Q(categories__slug__icontains='home-presentation-diy')
        )


class HomePresentation(Item):
    class Meta:
        proxy = True
        verbose_name = _('Home Presentation')
        verbose_name_plural = _('Home Presentations')
        ordering = ('-ordering', '-publication_date', 'title')

    def get_absolute_url(self):
        return "xxx/"

    @staticmethod
    def get_cms_qs():
        return HomePresentation.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = HomePresentationManager()


class CaseStudy(Item):
    def get_absolute_url(self):
        return "xxx/"

    class Meta:
        proxy = True
        ordering = ('-ordering', '-publication_date', 'title')

    @staticmethod
    def get_cms_qs():
        return Item.objects.filter(
            categories__slug__icontains='case-study'
        ).order_by('-ordering', '-publication_date', 'title')


class ConventionManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(
            categories__slug__icontains='convention'
        )


class Convention(Item):
    class Meta:
        proxy = True
        verbose_name = _('Convention')
        verbose_name_plural = _('Conventions')
        ordering = ('-ordering', '-publication_date', 'title')

    @staticmethod
    def get_cms_qs():
        return Convention.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = ConventionManager()


class CourseManager(models.Manager):
    def get_queryset(self):
        return Item.objects.filter(
            categories__slug__icontains='course'
        )


class Course(Item):
    class Meta:
        proxy = True
        verbose_name = _('Course')
        verbose_name_plural = _('Courses')
        ordering = ('-ordering', '-publication_date', 'title')

    @staticmethod
    def get_cms_qs():
        return Course.objects.filter(status=1).order_by('-ordering', '-publication_date', 'title')

    objects = CourseManager()


class Gallery(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True
    )
    description = RichTextField(
        verbose_name=_('description'),
        blank=True
    )

    def __unicode__(self):
        return u'{}'.format(self.name)

    class Meta:
        verbose_name = _(u'Gallery')
        verbose_name_plural = _(u'Galleries')
        ordering = ('ordering', 'name')


class GalleryImage(DateModel, UserModel, OrderedModel, StatusModel, SEOModel):
    name = models.CharField(
        verbose_name=_('name'),
        max_length=255,
    )
    slug = models.SlugField(
        verbose_name=_('slug'),
        max_length=255,
        unique=True
    )
    gallery = models.ForeignKey(
        Gallery,
        verbose_name=_('gallery'),
        related_name='gallery_images'
    )
    image = models.ImageField(
        upload_to='galleries',
        verbose_name=_('image'),
    )
    cropping = ImageRatioField(
        'image',
        settings.PRODUCT_GALLERYIMAGE_CROPPING_RATIO,
        free_crop=True
    )
    thumbnail = ImageRatioField(
        'image',
        settings.PRODUCT_GALLERYIMAGE_THUMBNAIL_RATIO,
        size_warning=True
    )

    def __unicode__(self):
        return u'{0}: {1}'.format(self.gallery, self.name)

    class Meta:
        verbose_name = _(u'Gallery Image')
        verbose_name_plural = _(u'Gallery Images')
        ordering = ('ordering', 'name')

    def get_cropped_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': settings.PRODUCT_GALLERYIMAGE_CROPPING_RATIO.split('x'),
                'box': self.cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False

    def get_cropped_full_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': settings.PRODUCT_GALLERYIMAGE_CROPPING_RATIO_FULL.split('x'),
                'box': self.cropping,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False

    def get_thumbnail(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': settings.PRODUCT_GALLERYIMAGE_THUMBNAIL_RATIO.split('x'),
                'box': self.thumbnail,
                'crop': True,
                'detail': True,
            }).url
        else:
            return False
