# coding: utf-8

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import Http404
from django.shortcuts import render_to_response, redirect
from django.utils.translation import ugettext as _
from django.views.generic import ListView, TemplateView, DetailView

from isolmant.product import models


class ProductAjaxLoader(ListView):
    model = models.Product
    template_name = "/product/includes/_ajax_product.html"
    paginate_by = 10

    def get_queryset(self, **kwargs):
        return models.Product.objects.filter(status=1).prefetch_related(
            "subcategory__application__category",
            "subcategory__application",
            "subcategory",
        )


class ProductDetailView(DetailView):
    template_name = '/product/product_detail.html'
    model = models.Product
    queryset = models.Product.objects.filter(status=1)


class SolutionAjaxLoader(ListView):
    model = models.Solution
    template_name = "/product/includes/_ajax_solution.html"
    paginate_by = 30

    def dispatch(self, request, *args, **kwargs):
        if request.is_ajax:
            return super(SolutionAjaxLoader, self).dispatch(request, *args, **kwargs)
        else:
            pass  # should take the user back to homepage or something similar

    def get_queryset(self, **kwargs):
        return models.Solution.objects.filter(status=1)


class SolutionDetailView(DetailView):
    template_name = '/product/solution_detail.html'
    model = models.Solution
    queryset = models.Solution.objects.filter(status=1)

    def get_queryset(self):
        qs = super(SolutionDetailView, self).get_queryset()
        qs = qs.filter(slug__isnull=False).exclude(slug='')

        return qs

    def get_object(self, slug, slug_field):
        queryset = self.get_queryset()
        queryset = queryset.filter(**{slug_field: slug})
        try:
            return queryset.get()
        except queryset.model.DoesNotExist:
            return None

    def get(self, request, *args, **kwargs):
        slug = self.kwargs.get('slug', None)

        # se trova la pagina
        if slug is not None:
            language_code = request.LANGUAGE_CODE
            self.object = self.get_object(slug, "slug")
            if self.object:
                context = self.get_context_data(object=self.object)
                return self.render_to_response(context)

            # se non trova la pagina prova con le altre lingue modificando lo slug
            for lang in settings.LANGUAGES:
                if lang[0] != language_code:
                    slug_field = "slug_{}".format(lang[0])
                    obj = self.get_object(slug, slug_field)
                    if obj:
                        return redirect(reverse('solution_detail', kwargs={'slug': obj.slug}))
        raise Http404(_("No solution found matching the query"))

    def get_context_data(self, **kwargs):
        ctx = super(SolutionDetailView, self).get_context_data(**kwargs)
        ctx['similar_solutions'] = self.model.objects.filter(
            application=self.object.application
        ).exclude(
            id=self.object.id
        )
        return ctx


class NewsDetailView(DetailView):
    model = models.News
    template_name = '/product/news_detail.html'


class ReferenceDetailView(DetailView):
    template_name = '/product/reference_detail.html'
    model = models.Reference


class BlogDetailView(DetailView):
    model = models.Blog
    template_name = '/product/blog_detail.html'
